@@ -4,126 +4,58 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2017 Insecure.Com LLC ("The Nmap  *
+ * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
  * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
+ * This program is distributed under the terms of the Nmap Public Source   *
+ * License (NPSL). The exact license text applying to a particular Nmap    *
+ * release or source code control revision is contained in the LICENSE     *
+ * file distributed with that version of Nmap or source code control       *
+ * revision. More Nmap copyright/legal information is available from       *
+ * https://nmap.org/book/man-legal.html, and further information on the    *
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+ * header summarizes some key points from the Nmap license, but is no      *
+ * substitute for the actual license text.                                 *
  *                                                                         *
- * o Integrates source code from Covered Software.                         *
+ * Nmap is generally free for end users to download and use themselves,    *
+ * including commercial use. It is available from https://nmap.org.        *
  *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
+ * The Nmap license generally prohibits companies from using and           *
+ * redistributing Nmap in commercial products, but we sell a special Nmap  *
+ * OEM Edition with a more permissive license and special features for     *
+ * this purpose. See https://nmap.org/oem/                                 *
  *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
+ * If you have received a written Nmap license agreement or contract       *
+ * stating terms other than these (such as an Nmap OEM license), you may   *
+ * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@nmap.com for further *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
+ * The official Nmap Windows builds include the Npcap software             *
+ * (https://npcap.com) for packet capture and transmission. It is under    *
+ * separate license terms which forbid redistribution without special      *
+ * permission. So the official Nmap Windows builds may not be              *
+ * redistributed without special permission (such as an Nmap OEM           *
+ * license).                                                               *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
  * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
+ * and add new features.  You are highly encouraged to submit your         *
+ * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+ * for possible incorporation into the main distribution. Unless you       *
+ * specify otherwise, it is understood that you are offering us very       *
+ * broad rights to use your submissions as described in the Nmap Public    *
+ * Source License Contributor Agreement. This is important because we      *
+ * fund the project by selling licenses with various terms, and also       *
+ * because the inability to relicense code has caused devastating          *
+ * problems for other Free Software projects (such as KDE and NASM).       *
  *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
+ * The free version of Nmap is distributed in the hope that it will be     *
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+ * indemnification and commercial support are all available through the    *
+ * Npcap OEM program--see https://nmap.org/oem/                            *
  *                                                                         *
  ***************************************************************************/
 
@@ -142,7 +74,6 @@
 #include <openssl/bn.h>
 #include <openssl/bio.h>
 #include <openssl/pem.h>
-#include <openssl/rsa.h>
 #include <openssl/ssl.h>
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
@@ -158,12 +89,31 @@
 #define X509_get0_notAfter X509_get_notAfter
 #endif
 
+/* OPENSSL_API_LEVEL per OpenSSL 3.0: decimal MMmmpp */
+#ifndef OPENSSL_API_LEVEL
+# if OPENSSL_API_COMPAT < 0x900000L
+#  define OPENSSL_API_LEVEL (OPENSSL_API_COMPAT)
+# else
+#  define OPENSSL_API_LEVEL \
+     (((OPENSSL_API_COMPAT >> 28) & 0xF) * 10000  \
+      + ((OPENSSL_API_COMPAT >> 20) & 0xFF) * 100 \
+      + ((OPENSSL_API_COMPAT >> 12) & 0xFF))
+# endif
+#endif
 
-extern "C"
-{
-#include "lua.h"
-#include "lauxlib.h"
-}
+#if OPENSSL_API_LEVEL >= 30000
+#include <openssl/core_names.h>
+/* Deprecated in OpenSSL 3.0 */
+#define SSL_get_peer_certificate SSL_get1_peer_certificate
+#else
+#include <openssl/rsa.h>
+#endif
+
+
+/* struct tm */
+#include <time.h>
+
+#include "nse_lua.h"
 
 #include "nse_nsock.h"
 
@@ -172,12 +122,11 @@ struct cert_userdata {
   int attributes_table;
 };
 
-/* from nse_openssl.cc */
 typedef struct bignum_data {
   BIGNUM * bn;
+  bool should_free;
 } bignum_data_t;
 
-
 SSL *nse_nsock_get_ssl(lua_State *L);
 
 /* This is a reference to a table that will be used as the metatable for
@@ -265,7 +214,7 @@ static void x509_name_to_table(lua_State *L, X509_NAME *name)
 {
   int i;
 
-  lua_newtable(L);
+  lua_createtable(L, 0, X509_NAME_entry_count(name));
 
   for (i = 0; i < X509_NAME_entry_count(name); i++) {
     X509_NAME_ENTRY *entry;
@@ -288,7 +237,7 @@ static bool x509_extensions_to_table(lua_State *L, const STACK_OF(X509_EXTENSION
   if (sk_X509_EXTENSION_num(exts) <= 0)
     return false;
 
-  lua_newtable(L);
+  lua_createtable(L, sk_X509_EXTENSION_num(exts), 0);
 
   for (int i = 0; i < sk_X509_EXTENSION_num(exts); i++) {
     ASN1_OBJECT *obj;
@@ -299,7 +248,8 @@ static bool x509_extensions_to_table(lua_State *L, const STACK_OF(X509_EXTENSION
     ext = sk_X509_EXTENSION_value(exts, i);
     obj = X509_EXTENSION_get_object(ext);
 
-    lua_newtable(L);
+#define NSE_NUM_X509_EXTENSION_FIELDS 3
+    lua_createtable(L, 0, NSE_NUM_X509_EXTENSION_FIELDS);
     char objname[256];
     long len = 0;
     len = OBJ_obj2txt(objname, 256, obj, 0);
@@ -324,7 +274,7 @@ static bool x509_extensions_to_table(lua_State *L, const STACK_OF(X509_EXTENSION
     }
     BIO_free_all(out);
 
-    lua_seti(L, -2, i+1);
+    lua_rawseti(L, -2, i+1);
   }
 
   return true;
@@ -418,20 +368,21 @@ static int time_to_tm(const ASN1_TIME *t, struct tm *result)
    that the wday and yday fields are not present. */
 static void tm_to_table(lua_State *L, const struct tm *tm)
 {
-  lua_newtable(L);
+#define NSE_NUM_TM_FIELDS 6
+  lua_createtable(L, 0, NSE_NUM_TM_FIELDS);
 
-  lua_pushnumber(L, tm->tm_year);
+  lua_pushinteger(L, tm->tm_year);
   lua_setfield(L, -2, "year");
   /* Lua uses one-indexed months. */
-  lua_pushnumber(L, tm->tm_mon + 1);
+  lua_pushinteger(L, tm->tm_mon + 1);
   lua_setfield(L, -2, "month");
-  lua_pushnumber(L, tm->tm_mday);
+  lua_pushinteger(L, tm->tm_mday);
   lua_setfield(L, -2, "day");
-  lua_pushnumber(L, tm->tm_hour);
+  lua_pushinteger(L, tm->tm_hour);
   lua_setfield(L, -2, "hour");
-  lua_pushnumber(L, tm->tm_min);
+  lua_pushinteger(L, tm->tm_min);
   lua_setfield(L, -2, "min");
-  lua_pushnumber(L, tm->tm_sec);
+  lua_pushinteger(L, tm->tm_sec);
   lua_setfield(L, -2, "sec");
   /* Omit tm_wday and tm_yday. */
 }
@@ -459,7 +410,8 @@ static void asn1_time_to_obj(lua_State *L, const ASN1_TIME *s)
    from asn1_time_to_obj. */
 static void x509_validity_to_table(lua_State *L, X509 *cert)
 {
-  lua_newtable(L);
+#define NSE_NUM_VALIDITY_FIELDS 2
+  lua_createtable(L, 0, NSE_NUM_VALIDITY_FIELDS);
 
   asn1_time_to_obj(L, X509_get0_notBefore(cert));
   lua_setfield(L, -2, "notBefore");
@@ -497,7 +449,7 @@ static const char *pkey_type_to_string(int type)
     return "dsa";
   case EVP_PKEY_DH:
     return "dh";
-#ifdef HAVE_OPENSSL_EC
+#ifdef EVP_PKEY_EC
   case EVP_PKEY_EC:
     return "ec";
 #endif
@@ -507,13 +459,45 @@ static const char *pkey_type_to_string(int type)
 }
 
 int lua_push_ecdhparams(lua_State *L, EVP_PKEY *pubkey) {
-#ifdef HAVE_OPENSSL_EC
+#if OPENSSL_API_LEVEL >= 30000
+  char tmp[64] = {0};
+  size_t len = 0;
+  /* This structure (ecdhparams.curve_params) comes from tls.lua */
+  lua_createtable(L, 0, 1); /* ecdhparams */
+  lua_createtable(L, 0, 2); /* curve_params */
+  if (EVP_PKEY_get_utf8_string_param(pubkey, OSSL_PKEY_PARAM_GROUP_NAME,
+        tmp, sizeof(tmp), &len)) {
+    lua_pushlstring(L, tmp, len);
+    lua_setfield(L, -2, "curve");
+    lua_pushliteral(L, "namedcurve");
+    lua_setfield(L, -2, "ec_curve_type");
+  }
+  else if (EVP_PKEY_get_utf8_string_param(pubkey, OSSL_PKEY_PARAM_EC_FIELD_TYPE,
+        tmp, sizeof(tmp), &len)) {
+    /* According to RFC 5480 section 2.1.1, explicit curves must not be used with
+       X.509. This may change in the future, but for now it doesn't seem worth it
+       to add in code to extract the extra parameters. */
+    if (0 == strncmp(tmp, "prime-field", len)) {
+      lua_pushliteral(L, "explicit_prime");
+    }
+    else if (0 == strncmp(tmp, "characteristic-two-field", len)) {
+      lua_pushliteral(L, "explicit_char2");
+    }
+    else {
+      /* Something weird happened. */
+      lua_pushlstring(L, tmp, len);
+    }
+    lua_setfield(L, -2, "ec_curve_type");
+  }
+  lua_setfield(L, -2, "curve_params");
+  return 1;
+#elif !defined(OPENSSL_NO_EC)
   EC_KEY *ec_key = EVP_PKEY_get1_EC_KEY(pubkey);
   const EC_GROUP *group = EC_KEY_get0_group(ec_key);
   int nid;
   /* This structure (ecdhparams.curve_params) comes from tls.lua */
-  lua_newtable(L); /* ecdhparams */
-  lua_newtable(L); /* curve_params */
+  lua_createtable(L, 0, 1); /* ecdhparams */
+  lua_createtable(L, 0, 2); /* curve_params */
   if ((nid = EC_GROUP_get_curve_name(group)) != 0) {
     lua_pushstring(L, OBJ_nid2sn(nid));
     lua_setfield(L, -2, "curve");
@@ -591,7 +575,8 @@ static int parse_ssl_cert(lua_State *L, X509 *cert)
   udata = (struct cert_userdata *) lua_newuserdata(L, sizeof(*udata));
   udata->cert = cert;
 
-  lua_newtable(L);
+#define NSE_NUM_CERT_FIELDS 7
+  lua_createtable(L, 0, NSE_NUM_CERT_FIELDS);
 
   subject = X509_get_subject_name(cert);
   if (subject != NULL) {
@@ -631,15 +616,17 @@ static int parse_ssl_cert(lua_State *L, X509 *cert)
   if (pubkey == NULL) {
     lua_pushnil(L);
     lua_pushfstring(L, "Error parsing cert: %s", ERR_error_string(ERR_get_error(), NULL));
+    X509_free(cert);
     return 2;
   }
-  lua_newtable(L);
+#define NSE_NUM_PKEY_FIELDS 4
+  lua_createtable(L, 0, NSE_NUM_PKEY_FIELDS);
 #if HAVE_OPAQUE_STRUCTS
   pkey_type = EVP_PKEY_base_id(pubkey);
 #else
   pkey_type = EVP_PKEY_type(pubkey->type);
 #endif
-#ifdef HAVE_OPENSSL_EC
+#ifdef EVP_PKEY_EC
   if (pkey_type == EVP_PKEY_EC) {
     lua_push_ecdhparams(L, pubkey);
     lua_setfield(L, -2, "ecdhparams");
@@ -647,18 +634,49 @@ static int parse_ssl_cert(lua_State *L, X509 *cert)
   else
 #endif
   if (pkey_type == EVP_PKEY_RSA) {
+#if OPENSSL_API_LEVEL < 30000
     RSA *rsa = EVP_PKEY_get1_RSA(pubkey);
-    bignum_data_t * data = (bignum_data_t *) lua_newuserdata( L, sizeof(bignum_data_t));
-    luaL_getmetatable( L, "BIGNUM" );
-    lua_setmetatable( L, -2 );
-  #if HAVE_OPAQUE_STRUCTS
-    const BIGNUM *n, *e, *d;
-    RSA_get0_key(rsa, &n, &e, &d);
-    data->bn = (BIGNUM*) e;
-  #else
-    data->bn = rsa->e;
-  #endif
-    lua_setfield(L, -2, "exponent");
+    if (rsa) {
+#endif
+      /* exponent */
+      bignum_data_t * data = (bignum_data_t *) lua_newuserdata( L, sizeof(bignum_data_t));
+      luaL_getmetatable( L, "BIGNUM" );
+      lua_setmetatable( L, -2 );
+#if HAVE_OPAQUE_STRUCTS
+#if OPENSSL_API_LEVEL < 30000
+      const BIGNUM *n = NULL, *e = NULL;
+      data->should_free = false;
+      RSA_get0_key(rsa, &n, &e, NULL);
+#else
+      BIGNUM *n = NULL, *e = NULL;
+      data->should_free = true;
+      EVP_PKEY_get_bn_param(pubkey, OSSL_PKEY_PARAM_RSA_E, &e);
+      EVP_PKEY_get_bn_param(pubkey, OSSL_PKEY_PARAM_RSA_N, &n);
+#endif
+      data->bn = (BIGNUM*) e;
+#else
+      data->bn = rsa->e;
+#endif
+      lua_setfield(L, -2, "exponent");
+      /* modulus */
+      data = (bignum_data_t *) lua_newuserdata( L, sizeof(bignum_data_t));
+      luaL_getmetatable( L, "BIGNUM" );
+      lua_setmetatable( L, -2 );
+#if HAVE_OPAQUE_STRUCTS
+#if OPENSSL_API_LEVEL < 30000
+      data->should_free = false;
+#else
+      data->should_free = true;
+#endif
+      data->bn = (BIGNUM*) n;
+#else
+      data->bn = rsa->n;
+#endif
+      lua_setfield(L, -2, "modulus");
+#if OPENSSL_API_LEVEL < 30000
+      RSA_free(rsa);
+    }
+#endif
   }
   lua_pushstring(L, pkey_type_to_string(pkey_type));
   lua_setfield(L, -2, "type");
